/* Taken and adapted from the tetris plugin,
   (c) Clemens Kirchgatterer <clemens@thf.ath.cx> 2002   
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "display.h"
#include "x.xpm"
#include "o.xpm"

#define MINX  36
#define MINY  16
#define MAXX 685
#define MAXY 543

static int size, width, height, messageRgn;


static cOsdBase      *osd = NULL;
static tWindowHandle  win = -1;

static eDvbColor
DisplayColor(Color col) {
   switch (col) {
      case BGR: return (clrBackground);
      case RED: return (clrRed);
      case GRN: return (clrGreen);
      case BLU: return (clrBlue);
      case YEL: return (clrYellow);
      case MAG: return (clrMagenta);
      case CYN: return (clrCyan);
      case WHT: return (clrWhite);
      case BLK: return (clrBlack);
      case TRS: return (clrTransparent);
   }
   return (clrWhite);
}

void initBitmap(cBitmap &b, char*xpm[], eDvbColor col) {
   char yes='.'; //read xpm to understand this
   eDvbColor bg=DisplayColor(BGR);
   for (int x=4;x<=83;x++) {
      for (int y=0;y<=80;y++) {
         b.SetPixel(x,y, (xpm[x][y] == yes) ? col : bg);
      }
   }
}

void initXBitmap( cBitmap &b) {
   initBitmap(b,x_xpm, DisplayColor(GRN));
}

void initOBitmap( cBitmap &b) {
   initBitmap(b,o_xpm, DisplayColor(RED));
}

void initEmptyBitmap(cBitmap &b) {
   b.Fill(0,0,b.Width(), b.Height(), DisplayColor(BGR));
}

int 
DisplayOpen(int x, int y, int w, int h) {
   int s=2;
   size    = s*4+4;
   //width   = 10*size;
   width=w;  //must be a product of 4
   height=h+cOsd::LineHeight()+(cOsd::LineHeight()%4); //adding space for messages
   messageRgn=h+1;


   int xpos, ypos;

   if (osd) return (-1);
   xpos = MINX+(int)((MAXX-width-MINX)*((x+9.0)/18.0));
   ypos = MINY+(int)((MAXY-height-MINY)*((y+9.0)/18.0));
   osd = cOsd::OpenRaw(xpos, ypos);
   if (!osd) {
      cOsd::Close();
      osd = cOsd::OpenRaw(xpos, ypos);
      if (!osd) {
         return (-1);
      }
   }
   win = osd->Create(0, 0, width, height, 4);
   osd->AddColor(clrBackground);
   osd->AddColor(clrTransparent);
   osd->AddColor(clrGreen);
   osd->AddColor(clrYellow);
   osd->AddColor(clrMagenta);
   osd->AddColor(clrBlue);
   osd->AddColor(clrWhite);
   osd->AddColor(clrCyan);
   osd->AddColor(clrBlack);
   osd->AddColor(clrRed);
   osd->SetFont(fontFix);
   osd->Show(win);
   osd->Flush();
   
   //Make the last line, which is for messages, fully transparent
   DisplayClearMessage();   
   osd->Flush();
   
   return (0);
}

int
DisplayClose(void) {
   if (!osd) return (-1);
   osd->Clear(win);
   osd->Hide(win);
   osd->Flush();
   delete (osd);
   osd = NULL;
   return (0);
}

int
DisplayWidth(void) {
   return (width);
}

int
DisplayHeight(void) {
   return (height);
}

int
DisplayFontHeight(void) {
   return (24);
}

int
DisplayCharWidth(char c) {
   if (osd) {
      return (osd->Width(c));
   }
   return (-1);
}

int
DisplayStringWidth(const char *s) {
   if (osd) {
      return (osd->Width(s));
   }
   return (-1);
}

void
DisplayPrintText(int x, int y, const char *txt) {
   if (osd) {
      osd->Text(x, y, txt);
   }
}

void
DisplayDrawBlock(int x, int y, Color col) {
   int x1, y1, x2, y2;

   x1 = x*size;
   y1 = y*size;
   x2 = x*size+size;
   y2 = y*size+size;
   if (osd) {
      osd->Fill(x1, y1, x2, y2, DisplayColor(col), win);
   }
}

void DisplayDrawRect(int x1, int y1, int x2, int y2, Color col) {
   if (osd) {
      osd->Fill(x1, y1, x2, y2, DisplayColor(col), win);
   }
}

void DisplayDrawBitmap(int x, int y, const cBitmap &bitmap) {
   if (osd) {
      osd->SetBitmap(x, y, bitmap, win);
   }
}

void
DisplayClearBlock(int x, int y) {
   DisplayDrawBlock(x, y, BGR);
}

void DisplayMessage(const char *txt, Color col) {
   if (osd && txt) {
      int x = (width - osd->Width(txt)) / 2;
      if (x < 0)
         x = 0;      
      osd->Text(x, messageRgn, txt, clrWhite, DisplayColor(col));
   }
}

void DisplayClearMessage() {
   DisplayDrawRect(0, messageRgn, width, height, TRS);
}

void
DisplayUpdate() {
   if (osd) {
      osd->Flush();
   }
}
