/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _KAESEKAESTCHEN_H_
#define _KAESEKAESTCHEN_H_
#include "display.h"

class Point { //to fit to java peculiarities
public:
   Point() : x(0),y(0),m_null(true) {}
   Point(int xx, int yy) : x(xx),y(yy),m_null(false) {}
   void reset() {x=0;y=0;}
   void set(int xx, int yy) {x=xx;y=yy;}
   bool isNull() { return m_null; }
   void setNull(bool n) {m_null=n;}
   int x;
   int y;
private:
   bool m_null;
};

class PointArray { //emulate a bit a java array, but size is fixed to 9
public:
   PointArray() : m_index(-1) {}
   const Point& operator[](int index) { return (0<=index<=m_index) ? m_p[index] : m_p[0]; }
   void add(Point p) { if (m_index<8) m_p[++m_index]=p; }
   bool isNull() { return m_index == -1; }
   void setNull() {m_index=-1;}
   int length() { return m_index+1;}
private:
   Point m_p[9];
   int m_index;
};

//see comment about German naming scheme in the cpp file
class Kaesekaestchen {
public:
   Kaesekaestchen();
   ~Kaesekaestchen();
   void Start(int x, int y);
   void Stop();
   //methods to control the game
   void set(int xfield, int yfield);
   void increaseDifficulty();
   void decreaseDifficulty();
   void toggleBeginner();
   void resetGame();
   void pressedOk();
private:
   cBitmap xBitmap;
   cBitmap oBitmap;
   cBitmap emptyBitmap;
   //functions
   void reinit();
   void startKK();
   void drawLines();
                    
   void cleanScreen();
   bool check2er(const int br[3][3],bool wer);
   bool ende (const int br[3][3]);
   void setze(int x,int y,bool wer);
   void setze(Point p,bool wer) { setze(p.x,p.y,wer); }
   bool legal(int x,int y,bool wer);
   bool legal(Point p,bool wer) {return legal(p.x,p.y,wer);}
   void endSequenz();
   void ziehe();
   bool setze2er();
   //int** kopiereBrett();
   bool setzeProblem1();
   void setzeInBeliebigeEcke();
   void setzeBeliebig();
   void setze1zu4();
   void setzeProblem1Umgekehrt();
   void erzeugeAlt();
   int nextInt(int n);
   int gib0oder1();
   //variables
   int brett[3][3];//-1;0;1
   //bool moeglich[3][3];
   Point mussHin;//nur von 0 bis 2 zu benutzen
   int anzMussHin;//Anzahl der 2er,die durch Setzen auf mussHin entstehen
   PointArray alt; //Feld mit allen freien Punkten
   bool beginner;
   int stufe;
   int sieger;
   bool weristdran;
   int z;
   bool schonInitiiert;
   bool resetting;
   int anzGewonneneSpieleComputer;
   int anzGewonneneSpieleSpieler;
   int anzSpieleGesamt;
   bool dirtyScreen;
   bool messageDisplaying;
   //Hilfsvariablen fr bestimmte Methoden
   int anzErzeugeAlt;
   int anzMHSetze2er;
   Point MHSetze2er;
   bool varZug1Ziehe;
   bool varZug1BeginnerZiehe;
   bool spielBeendet;
};

#endif //_KAESEKAESTCHEN_H_

