/*
 * tictactoe.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */
 
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <vdr/plugin.h>
#include "i18n.h"
#include "kaesekaestchen.h"

static const char *VERSION        = "0.1";
static const char *DESCRIPTION    = "A little game";
static const char *MAINMENUENTRY  = "Tic Tac Toe";

// default values
static int  x = -8; // x position
static int  y = -8; // y position

class KaesekaestchenGame : public cOsdObject {
public:
   ~KaesekaestchenGame()   { kk.Stop(); }
   void Show(void) { kk.Start(x, y); }
   eOSState ProcessKey(eKeys Key);
private:
   Kaesekaestchen kk;
};

class cPluginTictactoe : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginTictactoe(void);
  virtual ~cPluginTictactoe();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual bool Start(void);
  virtual const char *MainMenuEntry(void) { return tr(MAINMENUENTRY); }
  virtual cOsdObject *MainMenuAction(void);
//  virtual cMenuSetupPage *SetupMenu(void);
//  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginTictactoe::cPluginTictactoe(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginTictactoe::~cPluginTictactoe()
{
  // Clean up after yourself!
}


bool cPluginTictactoe::Start(void)
{
  // Start any background activities the plugin shall perform.  
  RegisterI18n(Phrases);
  return true;
}

cOsdObject *cPluginTictactoe::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return new KaesekaestchenGame;
}

/*cMenuSetupPage *cPluginTictactoe::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginTictactoe::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}*/

eOSState KaesekaestchenGame::ProcessKey(eKeys Key) {
   switch (Key) {
      case k1:
         kk.set(0,0);
         break;
      case k2:
         kk.set(1,0);
         break;
      case k3:
         kk.set(2,0);
         break;
      case k4:
         kk.set(0,1);
         break;
      case k5:
         kk.set(1,1);
         break;
      case k6:
         kk.set(2,1);
         break;
      case k7:
         kk.set(0,2);
          break;
      case k8:
         kk.set(1,2);
         break;
      case k9:
         kk.set(2,2);
         break;
      case k0:
         kk.resetGame();
         break;
      case kUp:
         kk.increaseDifficulty();
         break;
      case kDown:
         kk.decreaseDifficulty();
         break;
      case kLeft:
      case kRight:
         kk.toggleBeginner();
         break;
      case kOk: 
         kk.pressedOk();
         break;
      case kBack:
         return (osEnd);
      default: 
         break;
   }
   return (osContinue);
}


VDRPLUGINCREATOR(cPluginTictactoe); // Don't touch this!
